from collections import namedtuple
from trolley.exceptions.invalidFieldException import InvalidFieldException
import trolley.configuration

from trolley.types.meta import Meta

class InvoiceLineGateway(object):
    """
    Trolley InvoiceLine processor
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    """ Creates a new InvoiceLine. Only provide the line item in the body, and provide invoiceId separately.  """
    def create(self, invoice_id, body,):
        if invoice_id is None:
            raise InvalidFieldException("Invoice ID cannot be None.")
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        if not isinstance(body, list):
            raise InvalidFieldException("Body must be of type list")

        if "invoiceId" in body:
            del body['invoiceId']
        
        payload = {
            'invoiceId' : invoice_id,
            'lines' : body
        }

        endpoint = f'/v1/invoices/create-lines/'
        response = trolley.Configuration.client(
            self.config).post(endpoint, payload)

        temp_invoice = trolley.Invoice.factory(response)
        invoice = namedtuple("Invoice", temp_invoice.keys())(*temp_invoice.values())
        return invoice

    """ Update an Invoice Line. """
    def update(self, invoice_id, body):
        if invoice_id is None:
            raise InvalidFieldException("Invoice ID cannot be None.")
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        if not isinstance(body, list):
            raise InvalidFieldException("Body must be of type list")
        
        if "invoiceId" in body:
            del body['invoiceId']
        
        payload = {
            'invoiceId' : invoice_id,
            'lines' : body
        }

        endpoint = f'/v1/invoices/update-lines'
        response = trolley.Configuration.client(
            self.config).post(endpoint, payload)
        temp_invoice = trolley.Invoice.factory(response)
        invoice = namedtuple("Invoice", temp_invoice.keys())(*temp_invoice.values())
        return invoice

    """ Delete Invoice Lines. Provide a list of str in invoice_line_ids """
    def delete(self, invoice_id, invoice_line_ids):
        if invoice_id is None:
            raise InvalidFieldException("Invoice ID cannot be None.")
        if invoice_line_ids is None:
            raise InvalidFieldException("Invoice Line IDs cannot be None.")
        if not isinstance(invoice_line_ids, list):
            raise InvalidFieldException("Invoice Line IDs cannot be None.")
        
        payload = {
            'invoiceId' : invoice_id,
            'invoiceLineIds' : invoice_line_ids
        }

        endpoint = f'/v1/invoices/delete-lines'
        response = trolley.Configuration.client(
            self.config).post(endpoint, payload)

        return response['ok']



from collections import namedtuple
from trolley.exceptions.invalidFieldException import InvalidFieldException
import trolley.configuration

from trolley.types.meta import Meta
from trolley.types.payment import Payment

class PaymentGateway(object):
    """
    Trolley Payment processor
    Creates and manages transactions
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    """
        Retrieve a payment
            A payment_id and batch_id are required::
            payment.find('P-dejrtewdsj',B-fjeracjmuflh')
        """
    def find(self, payment_id, batch_id):
        if payment_id is None:
            raise InvalidFieldException("Payment id cannot be None.")
        if batch_id is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batch_id + '/payments/' + payment_id
        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint)
        temppayment = Payment.factory(response)
        payment = namedtuple("Payment", temppayment.keys())(*temppayment.values())
        return payment

    """
        Create a payment
            A body and batch_id are required::
            payment.create(
               {"recipient":{"id":"R-91XNJBKM30F06"},"sourceAmount":"100.10",
               "memo":"Freelance payment"}
        """
    def create(self, body, batch_id):
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        elif batch_id is None:
            raise InvalidFieldException("Batch ID cannot be None.")
        endpoint = '/v1/batches/' + batch_id + '/payments/'
        response = trolley.configuration.Configuration.client(
            self.config).post(endpoint, body)
        temppayment = Payment.factory(response)
        payment = namedtuple("Payment", temppayment.keys())(*temppayment.values())
        return payment

    """
        Update a payment
            A payment_id, batch_id, and body are required::
            payment.update('B-fjeracjmuflh',{"sourceAmount":"900.90"},'P-jddfjwojd')
        """
    def update(self, payment_id, body, batch_id):
        if payment_id is None:
            raise InvalidFieldException("Payment id cannot be None.")
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        endpoint = '/v1/batches/' + batch_id + '/payments/' + payment_id
        response = trolley.configuration.Configuration.client(
            self.config).patch(endpoint, body)
        temppayment = Payment.factory(response)
        payment = namedtuple("Payment", temppayment.keys())(*temppayment.values())
        return payment

    """
        Delete a payment
            A payment_id, batch_id are required::
            payment.delete('P-jddfjwojd','B-fjeracjmuflh',)
        """
    def delete(self, payment_id, batch_id):
        if payment_id is None:
            raise InvalidFieldException("Payment id cannot be None.")
        endpoint = '/v1/batches/' + batch_id + '/payments/' + payment_id
        trolley.configuration.Configuration.client(
            self.config).delete(endpoint)
        return True

    """ Lists all payments under a batch.
     This is basically an alias to the search() method. """
    def list_all_payments(self, batch_id):
        return self.search(batch_id)


    """ Search payments within a batch, whose id is provided, with a search term.
     This method returns a generator which auto paginates.
     You can use this generator in a foreach loop to sequentially go through all the 
      search results without needing to call this method again.
       
        For accessing specific pages, check the search_by_page() method.
         
          payment.search('B-fjeracjmuflh', 'keyword') """
    def search(self, batch_id, term=""):
        page = 0
        should_paginate = True
        while should_paginate:
            page+=1
            endpoint = f'/v1/batches/{batch_id}/payments?search={term}'
            params = f'&page={page}&pageSize=10'
            response = trolley.configuration.Configuration.client(self.config).get(endpoint+params)
            yield from self.__build_payments_from_response(response, False)

            if page < response["meta"]["pages"]:
                should_paginate = True
            else:
                should_paginate = False
    
    """ Search payments by providing a page number.
     You should use this function when you want to paginate manually.
      
       payment.search('B-fjeracjmuflh', 'keyword', 1, 20) """
    def search_by_page(self, batch_id, term="", page=1, page_size=10):
        endpoint = f'/v1/batches/{batch_id}/payments?search={term}'
        params = f'&page={page}&pageSize={page_size}'
        
        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint+params)
        
        return self.__build_payments_from_response(response, True)
        
    def __build_payments_from_response(self, response, include_meta=False):
        payments = []
        count = 0
        for payment in response['payments']:
            temppayment = Payment.factory(payment)
            newpayment = namedtuple("Payment", temppayment.keys())(*temppayment.values())
            payments.insert(count, newpayment)
            count = count + 1
        
        if include_meta:
            tempmeta = Meta.factory(response['meta'])
            payments.insert(count,namedtuple("Meta", tempmeta.keys())(*tempmeta.values()))

        return payments



from collections import namedtuple
from trolley.exceptions.invalidFieldException import InvalidFieldException
import trolley.configuration

from trolley.types.meta import Meta
from trolley.types.offline_payment import OfflinePayment

class OfflinePaymentGateway(object):
    """
    Trolley OfflinePayment processor
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    """ This method returns a generator which iterates through all the pages of Offline Payments.
     If you want to paginate manually, please use get_all_by_page() instead. """
    def get_all(self):
        page = 0
        should_paginate = True
        while should_paginate:
            page+=1
            endpoint = f'/v1/offline-payments?page={page}&pageSize=10'
            response = trolley.configuration.Configuration.client(self.config).get(endpoint)
            yield from self.__build_offline_payments_from_response(response)

            if page < response["meta"]["pages"]:
                should_paginate = True
            else:
                should_paginate = False

    """ This method returns all offline payments but requires you to specify page number and page size manually.
     If you want to auto paginate, use get_all() method instead. """
    def get_all_by_page(self, page=1, pageSize=10):
            endpoint = f'/v1/offline-payments?page={page}&pageSize={pageSize}'
            response = trolley.configuration.Configuration.client(self.config).get(endpoint)
            return self.__build_offline_payments_from_response(response, True)

    """ Creates a new Offline Payment for a Recipient whose ID is provided """
    def create(self, recipient_id, body,):
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        elif recipient_id is None:
            raise InvalidFieldException("Recipient ID cannot be None.")
        endpoint = f'/v1/recipients/{recipient_id}/offlinePayments/'
        response = trolley.configuration.Configuration.client(
            self.config).post(endpoint, body)

        temppayment = OfflinePayment.factory(response)
        offlinepayment = namedtuple("OfflinePayment", temppayment.keys())(*temppayment.values())
        return offlinepayment

    """ Update an Offline Payment """
    def update(self, offline_payment_id, recipient_id, body):
        if offline_payment_id is None:
            raise InvalidFieldException("Offline Payment id cannot be None.")
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        if recipient_id is None:
            raise InvalidFieldException("Recipient ID cannot be None.")
        endpoint = f'/v1/recipients/{recipient_id}/offlinePayments/{offline_payment_id}'
        response = trolley.configuration.Configuration.client(
            self.config).patch(endpoint, body)
        temppayment = OfflinePayment.factory(response)
        offlinepayment = namedtuple("OfflinePayment", temppayment.keys())(*temppayment.values())
        return offlinepayment

    """ Delete an Offline Payment """
    def delete(self, recipient_id, offline_payment_id):
        if recipient_id is None:
            raise InvalidFieldException("Recipient ID cannot be None.")
        if offline_payment_id is None:
            raise InvalidFieldException("Offline Payment ID cannot be None.")
        
        endpoint = f'/v1/recipients/{recipient_id}/offlinePayments/{offline_payment_id}'
        response = trolley.configuration.Configuration.client(
            self.config).delete(endpoint)
        return response['ok']
        
    def __build_offline_payments_from_response(self, response, include_meta=False):
        offlinepayments = []
        count = 0
        for offlinepayment in response['offlinePayments']:
            temppayment = OfflinePayment.factory(offlinepayment)
            newpayment = namedtuple("OfflinePayment", temppayment.keys())(*temppayment.values())
            offlinepayments.insert(count, newpayment)
            count = count + 1
        
        if include_meta:
            tempmeta = Meta.factory(response['meta'])
            offlinepayments.insert(count,namedtuple("Meta", tempmeta.keys())(*tempmeta.values()))

        return offlinepayments



from collections import namedtuple
from trolley.exceptions.invalidFieldException import InvalidFieldException
import trolley.configuration

from trolley.types.meta import Meta

class InvoiceGateway(object):
    """
    Trolley Invoice processor
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    """ Create a new Invoice """
    def create(self, body,):
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        endpoint = f'/v1/invoices/create'
        response = trolley.Configuration.client(
            self.config).post(endpoint, body)

        temp_invoice = trolley.Invoice.factory(response)
        invoice = namedtuple("Invoice", temp_invoice.keys())(*temp_invoice.values())
        return invoice

    """ Update an Invoice. """
    def update(self, invoice_id, body):
        if invoice_id is None:
            raise InvalidFieldException("Invoice ID cannot be None.")
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        body["invoiceId"] = invoice_id
        endpoint = f'/v1/invoices/update'
        response = trolley.Configuration.client(
            self.config).post(endpoint, body)
        temp_invoice = trolley.Invoice.factory(response)
        invoice = namedtuple("Invoice", temp_invoice.keys())(*temp_invoice.values())
        return invoice

    """ Get details of an invoice whose ID is provided."""
    def get(self, invoice_id):
        if invoice_id is None:
            raise InvalidFieldException("Invoice ID cannot be None.")
        endpoint = f'/v1/invoices/get'
        response = trolley.Configuration.client(
            self.config).post(endpoint, {
                "invoiceId": invoice_id
            })

        temp_invoice = trolley.Invoice.factory(response)
        invoice = namedtuple("Invoice", temp_invoice.keys())(*temp_invoice.values())
        return invoice

    """ Search through invoices. This method auto paginates.
    If you want to paginate manually, use search_by_page() method instead.
      If you added pagination parameters in the body, they'll be ignored. """
    def search(self, body):
        page = 0
        should_paginate = True
        if "page" in body:
            del body['page']
        
        if "pageSize" in body:
            del body['pageSize']
        
        while should_paginate:
            page+=1
            endpoint = f'/v1/invoices/search'
            body['page'] = page

            response = trolley.Configuration.client(self.config).post(endpoint, body)
            yield from self.__build_invoice_from_response(response, False)

            if page < response["meta"]["pages"]:
                should_paginate = True
            else:
                should_paginate = False

    """ Search through invoices. This method paginates manually.
    If you want to auto paginate, use search() method instead.
      If you added pagination parameters in the body, they'll be ignored. Provide them as method parameter instead."""
    def search_by_page(self, body, page=1, pageSize=10):
        if "page" in body:
            del body['page']
        
        if "pageSize" in body:
            del body['pageSize']

        endpoint = f'/v1/invoices/search'
        body['page'] = page
        body['pageSize'] = pageSize

        response = trolley.Configuration.client(self.config).post(endpoint, body)
        return self.__build_invoice_from_response(response, True)

    """ Delete an Invoice """
    def delete(self, invoice_ids):
        if invoice_ids is None:
            raise InvalidFieldException("Invoice ID cannot be None.")
        
        body = {
            "invoiceIds": invoice_ids
        }

        endpoint = f'/v1/invoices/delete'
        response = trolley.Configuration.client(
            self.config).post(endpoint, body)

        return response['ok']
        
    def __build_invoice_from_response(self, response, include_meta=False):
        invoices = []
        count = 0
        for invoice in response['invoices']:
            tempinvoice = trolley.Invoice.factory(invoice)
            newinvoice = namedtuple("Invoice", tempinvoice.keys())(*tempinvoice.values())
            invoices.insert(count, newinvoice)
            count = count + 1
        
        if include_meta:
            temp_meta = Meta.factory(response['meta'])
            invoices.insert(count,namedtuple("Meta", temp_meta.keys())(*temp_meta.values()))

        return invoices



from collections import namedtuple
import trolley.configuration
from trolley.exceptions.invalidFieldException import InvalidFieldException
from trolley.types.batch import Batch
from trolley.types.batch_summary import BatchSummary
from trolley.types.meta import Meta

class BatchGateway(object):
    """
    Trolley Batch processor
    Creates and manages batches
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    """
        Retrieve a batch
            A batch_id is required::
            Batch.find('B-fjeracjmuflh')
        """
    def find(self, batchid):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batchid
        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint)
        tempbatch = Batch.factory(response)
        batch = namedtuple("Batch", tempbatch.keys())(*tempbatch.values())
        return batch

    """
    Create a batch
        A body is required::
        Batch.create(
            {"payments":[{"recipient":{"id":"R-SBAHDK3DK6M7SUEM"},
            "sourceAmount":"65","memo":"","sourceCurrency":"CAD"}]})
    """
    def create(self, body):
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        endpoint = '/v1/batches/'
        response = trolley.configuration.Configuration.client(
            self.config).post(endpoint, body)
        tempbatch = Batch.factory(response)
        batch = namedtuple("Batch", tempbatch.keys())(*tempbatch.values())
        return batch

    """
        Update a batch
            A batch_id and body are required::
            Batch.update('B-fjeracjmuflh',{"payments":[{"recipient":{"id":"R-3DF7FAF680739541",
            "email":"jsmith@example.com"},"sourceAmount":65,"memo":"Salary",
            "sourceCurrency":"CAD"}]})
        """
    def update(self, batchid, body):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        endpoint = '/v1/batches/' + batchid
        trolley.configuration.Configuration.client(
            self.config).patch(endpoint, body)
        return True

    """
        Delete a batch
            A batch_id is required::
            batch.delete('B-fjeracjmuflh')
        """
    def delete(self, batchid):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batchid
        trolley.configuration.Configuration.client(
            self.config).delete(endpoint)
        return True
    
    """
        Delete multiple batches
            A list of batch ids is required::
            batch.delete([
                'B-fjeracjmuflh',
                'B-fjeracabclh'
            ])
        """
    def delete_multiple(self, batchids):
        if batchids is None:
            raise InvalidFieldException("Batch IDs cannot be None.")
        endpoint = '/v1/batches/'
        trolley.configuration.Configuration.client(
            self.config).delete(endpoint, batchids)
        return True
    
    """ Lists all payments under a batch.
     This is basically an alias to the search() method. """
    def list_all_batches(self):
        return self.search()

    """ Search for a batch with a search term.
     This method returns a generator which auto paginates.
     You can use this generator in a foreach loop to sequentially go through all the 
      search results without needing to call this method again.
       
        For accessing specific pages, check the search_by_page() method. 
        
        batch.search('term')"""
    def search(self, term=""):
        page = 0
        should_paginate = True
        while should_paginate:
            page+=1
            endpoint = f'/v1/batches/?search={term}'
            params = f'&page={page}&pageSize=10'
            response = trolley.configuration.Configuration.client(self.config).get(endpoint+params)
            yield from self.__build_batches_from_response(response, False)

            if page < response["meta"]["pages"]:
                should_paginate = True
            else:
                should_paginate = False

    """ Search batches by providing a page number.
     You should use this function when you want to paginate manually.
      batch.search_by_page('term', 1, 10) """
    def search_by_page(self, term="", page=1, page_size=10):
        endpoint = f'/v1/batches?search={term}'
        params = f'&page={page}&pageSize={page_size}'

        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint+params)
        
        return self.__build_batches_from_response(response, True)
        
    
    def __build_batches_from_response(self, response, include_meta=False):
        batches = []
        count = 0
        for batch in response['batches']:
            tempbatch = Batch.factory(batch)
            newbatch = namedtuple("Batch", tempbatch.keys())(*tempbatch.values())
            batches.insert(count, newbatch)
            count = count + 1
        
        if include_meta:
            tempmeta = Meta.factory(response['meta'])
            batches.insert(count,namedtuple("Meta", tempmeta.keys())(*tempmeta.values()))
            
        return batches

    """
        Retrieve a summary about a specific batch
            A batch_id is required::
            batch.summary('B-fjeracjmuflh')
        """
    def summary(self, batchid):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batchid + '/summary'
        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint)
        tempbatchsummary = BatchSummary.factory(
            response)
        batchsummary = namedtuple("BatchSummary", tempbatchsummary.keys())(
            *tempbatchsummary.values())
        return batchsummary

    """
        Generate a quote for a batch
            A batch_id is required::
            batch.generate_quote('B-fjeracjmuflh')
        """
    def generate_quote(self, batchid):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batchid + '/generate-quote'
        response = trolley.configuration.Configuration.client(
            self.config).post(endpoint, {})
        tempbatch = Batch.factory(response)
        batch = namedtuple("Batch", tempbatch.keys())(*tempbatch.values())
        return batch

    """
        Process a batch
            A batch_id is required::
            batch.process_batch('B-fjeracjmuflh')
        """
    def process_batch(self, batchid):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batchid + '/start-processing'
        response = trolley.configuration.Configuration.client(
            self.config).post(endpoint, {})
        tempbatch = Batch.factory(response)
        batch = namedtuple("Batch", tempbatch.keys())(*tempbatch.values())
        return batch




from trolley.client import Client
from trolley.gateway import Gateway
from dotenv import dotenv_values

class Configuration(object):
    """
    Configuration registry
    """

    public_key = ''
    private_key = ''
    enviroment = ''

    def __init__(self, public_key="", private_key="", enviroment="production"):
        self.public_key = public_key
        self.private_key = private_key
        self.enviroment = Configuration.set_enviroment(enviroment)

    @staticmethod
    def gateway(public_key, private_key, enviroment = 'production'):
        return Gateway(Configuration(public_key, private_key, enviroment))

    @staticmethod
    def client(config):
        return Client.create(config)

    @staticmethod
    def instantiate():
        return Configuration()

    @staticmethod
    def set_public_key(public_key):
        """
        Set method for the public key
        """
        Configuration.public_key = public_key

    @staticmethod
    def get_public_key():
        """
        Get method for the public key
        """
        return Configuration.public_key

    @staticmethod
    def get_private_key():
        """
        Get method for the private key
        """
        return Configuration.private_key

    @staticmethod
    def set_private_key(private_key):
        """
        Get method for the private key
        """
        Configuration.private_key = private_key
    @staticmethod
    def set_api_base(enviroment):
        Configuration.enviroment = Configuration.set_enviroment(enviroment)

    @staticmethod
    def set_enviroment(enviroment):
        """
        Set method to change the enviroment
        """
        if enviroment == 'development':
            env_values = dotenv_values(".env")

            if len(env_values['SERVER_URL']) == 0:
                raise Exception("Environment selected is 'development' but SERVER_URL is empty in .env")
            else:
                return  env_values['SERVER_URL']
        
        return 'https://api.trolley.com'
    


from collections import namedtuple
from trolley.exceptions.invalidFieldException import InvalidFieldException
import trolley.configuration
from trolley.types.recipient_account import RecipientAccount


class RecipientAccountGateway(object):
    """
    Trolley RecipientAccount processor
    Creates and manages bank accounts
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    """
        [DEPRECATED] This method has been deprecated and will be removed in future releases.
        Retrieve all the recipient accounts
            A recipient_id is required::
            recipient_account.findAll('R-fjeracjmuflh')
        """
    def findAll(self, recipient_id):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id + '/accounts/'
        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint)

        recipientaccounts = []
        count = 0
        for recipientaccount in response['accounts']:
            recipaccount = RecipientAccount.factory(
                recipientaccount)
            newrecipientaccount = namedtuple(
                "RecipientAccount", recipaccount.keys())(*recipaccount.values())
            recipientaccounts.insert(count, newrecipientaccount)
            count = count + 1
        return recipientaccounts

    """
        Retrieve a recipient account
            A recipient_id and recipient_account_id are required::
            recipient_account.find('R-fjeracjmuflh','A-2DQMpN4jurTFn9gRxobx4C')
        """
    def find(self, recipient_id, recipient_account_id):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id + '/accounts/' + recipient_account_id
        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint)
        recipaccount = RecipientAccount.factory(
            response)
        recipientaccount = namedtuple(
            "RecipientAccount", recipaccount.keys())(*recipaccount.values())
        return recipientaccount

    """
        Create a recipient account
            A recipient_id and body are required::
            recipient_account.create('R-4625iLug2GKqKZG2WzAf3e',{"accountHolderName": "Acer" ...})
        """
    def create(self, recipient_id, body):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id + '/accounts'
        response = trolley.configuration.Configuration.client(
            self.config).post(endpoint, body)
        recipaccount = RecipientAccount.factory(
            response)
        recipientaccount = namedtuple(
            "RecipientAccount", recipaccount.keys())(*recipaccount.values())
        return recipientaccount

    """
        Update a recipient account
            A recipient_id, recipient_account_id, and body are required::
            recipient_account.update('R-fjeracjmuflh','A-2DQMpN4jurTFn9gRxobx4C',
            {"accountHolderName": "Acer Philips"})
        """
    def update(self, recipient_id, recipient_account_id, body):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id + '/accounts/' + recipient_account_id
        response = trolley.configuration.Configuration.client(
            self.config).patch(endpoint, body)
        recipaccount = RecipientAccount.factory(
            response)
        recipientaccount = namedtuple(
            "RecipientAccount", recipaccount.keys())(*recipaccount.values())
        return recipientaccount

    """
        Delete a recipient account
            A recipient_id and recipient_account_id are required::
            recipient_account.delete('R-fjeracjmuflh','A-2DQMpN4jurTFn9gRxobx4C')
        """
    def delete(self, recipient_id, recipient_account_id):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id + '/accounts/' + recipient_account_id
        trolley.configuration.Configuration.client(
            self.config).delete(endpoint)
        return True




import time
import hmac
import hashlib
import json
import requests

from trolley.exceptions.invalidFieldException import InvalidFieldException
from trolley.exceptions.unexpectedException import UnexpectedException
from trolley.exceptions.notFoundException import NotFoundException
from trolley.exceptions.authenticationException import AuthenticationException
from trolley.exceptions.authorizationException import AuthorizationException
from trolley.exceptions.invalidFieldException import InvalidFieldException
from trolley.exceptions.tooManyRequestsException import TooManyRequestsException
from trolley.exceptions.downForMaintenanceException import DownForMaintenanceException
from trolley.exceptions.malformedException import MalformedException
from trolley.exceptions.invalidStatusException import InvalidStatusException
from trolley.exceptions.invalidServerConnectionException import InvalidServerConnectionException


class Client(object):
    """
    A class that makes Client requests to the API.
    """

    def __init__(self, config):
        self.config = config

    @staticmethod
    def create(config):
        """
        Factory create method
        """
        return Client(config)

    def sendRequest(self,endpoint,method,body=""):
        try:
            timestamp = int(time.time())
            authorization = self.generate_authorization(timestamp, method, endpoint, self.config, body)

            headers = {'Content-Type': 'application/json',
                       'Authorization': authorization,
                       'X-PR-Timestamp': str(timestamp),
                       'Trolley-Source': 'python-sdk_1.0.0'
                       }
            
            if method == "GET":
                response = requests.get(self.config.enviroment + endpoint, headers=headers)
            elif method == "POST":
                response = requests.post(self.config.enviroment + endpoint, headers=headers, json=body)
            elif method == "PATCH":
                response = requests.patch(self.config.enviroment + endpoint, headers=headers, json=body)
            elif method == "DELETE":
                response = requests.delete(self.config.enviroment + endpoint, headers=headers, json=body)
            else:
                self.throw_status_code_exception(None, "Invalid Method")
            if response.status_code != 200 and response.status_code != 204:
                self.throw_status_code_exception(response.status_code, response.content.decode("utf-8"))

            data = json.loads(response.content.decode("utf-8"))
            return data

        except requests.exceptions.RequestException:
            raise InvalidServerConnectionException("Invalid Connection to the server")

    def get(self, endpoint):
        """
        Makes an HTTP GET request to the API
        """
        return self.sendRequest(endpoint,"GET")

    def post(self, endpoint, body):
        """
        Makes an HTTP POST request to the API
        """
        return self.sendRequest(endpoint,"POST",body)

    def patch(self, endpoint, body):
        """
        Makes an HTTP PATCH request to the API
        """
        return self.sendRequest(endpoint,"PATCH",body)   

    def delete(self, endpoint, body={}):
        """
        Makes an HTTP DELETE request to the API
        """
        return self.sendRequest(endpoint,"DELETE", body)

    def generate_authorization(self,timestamp, method, endpoint, config, body=''):
        """
        Generates an authorization signature for the request header
        """
        if body != '':
            body = json.dumps(body)
        message = str(timestamp) + '\n' + method + \
            '\n' + endpoint + '\n' + body + '\n'
        key = bytes(self.config.private_key.encode('utf-8'))
        signature = hmac.new(key, msg=message.encode(
            'utf-8'), digestmod=hashlib.sha256).hexdigest()
        return 'prsign ' + self.config.public_key + ':' + signature
    @staticmethod
    def throw_status_code_exception(status_code, message):
        """
        Throws an exception based on the type of error
        """
        if status_code == 400:
            raise MalformedException(message)
        elif status_code == 401:
            raise AuthenticationException(message)
        elif status_code == 403:
            raise AuthorizationException(message)
        elif status_code == 404:
            raise NotFoundException(message)
        elif status_code == 406:
            raise InvalidStatusException(message)
        elif status_code == 429:
            raise TooManyRequestsException(message)
        elif status_code == 500:
            raise InvalidServerConnectionException(message)
        elif status_code == 503:
            raise DownForMaintenanceException(message)
        else:
            raise UnexpectedException(
                'Unexpected HTTP_RESPONSE # ' + str(status_code) + " " + message)



import json

class BaseException(BaseException):
    """
    The Base Exception class.
    """

    def __init__(self, value):
        BaseException.error_array = json.loads(value)
        self.value = value

    def __str__(self):
        return repr(self.value)
    
    def get_error_array(self):
        return BaseException.error_array["errors"]



from trolley.recipient_gateway import RecipientGateway
from trolley.balances_gateway import BalancesGateway
from trolley.batch_gateway import BatchGateway
from trolley.payment_gateway import PaymentGateway
from trolley.offline_payment_gateway import OfflinePaymentGateway
from trolley.recipient_account_gateway import RecipientAccountGateway
from trolley.invoice_gateway import InvoiceGateway
from trolley.invoice_line_gateway import InvoiceLineGateway
from trolley.invoice_payment_gateway import InvoicePaymentGateway
import trolley.configuration


class Gateway(object):
    """
    Trolley gateway module
    """

    def __init__(self, config=None, **kwargs):
        if isinstance(config, trolley.configuration.Configuration):
            self.config = config
        else:
            self.config = trolley.configuration.Configuration(
                public_key=kwargs.get("public_key"),
                private_key=kwargs.get("private_key")
            )
        self.recipient = RecipientGateway(self, config)
        self.recipient_account = RecipientAccountGateway(self, config)
        self.batch = BatchGateway(self, config)
        self.payment = PaymentGateway(self, config)
        self.offline_payment = OfflinePaymentGateway(self, config)
        self.balances = BalancesGateway(self, config)
        self.invoice = InvoiceGateway(self, config)
        self.invoice_line = InvoiceLineGateway(self, config)
        self.invoice_payment = InvoicePaymentGateway(self, config)


from collections import namedtuple
from trolley.exceptions.invalidFieldException import InvalidFieldException
import trolley.configuration

from trolley.types.meta import Meta

class InvoicePaymentGateway(object):
    """
    Trolley Invoice Payment processor
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    """ Creates a new Invoice Payment.  """
    def create(self, body):
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        if not isinstance(body, list):
            raise InvalidFieldException("Body must be of type list")
        
        payload = {
            'ids' : body
        }

        endpoint = f'/v1/invoices/payment/create'
        response = trolley.Configuration.client(
            self.config).post(endpoint, payload)

        temp_invoice_payment = trolley.InvoicePayment.factory(response)
        invoice = namedtuple("InvoicePayment", temp_invoice_payment.keys())(*temp_invoice_payment.values())
        return invoice

    """ Update an Invoice Payment. """
    def update(self, body):
        if body is None:
            raise InvalidFieldException("Body cannot be None.")

        endpoint = f'/v1/invoices/payment/update'
        response = trolley.Configuration.client(
            self.config).post(endpoint, body)

        return response['ok']

    """ Remove the association between a payment and an invoice.
    Provide a list of str in invoice_line_ids """
    def delete(self, payment_id, invoice_line_ids):
        if payment_id is None:
            raise InvalidFieldException("Payment ID cannot be None.")
        if invoice_line_ids is None:
            raise InvalidFieldException("Invoice Line IDs cannot be None.")
        if not isinstance(invoice_line_ids, list):
            raise InvalidFieldException("Invoice Line IDs must be of type list.")
        
        payload = {
            'paymentId' : payment_id,
            'invoiceLineIds' : invoice_line_ids
        }

        endpoint = f'/v1/invoices/payment/delete'
        response = trolley.Configuration.client(
            self.config).post(endpoint, payload)

        return response['ok']
    
    """ Search for Invoice Payments. Provide a list of str of payment or invoice IDs in the argument ids """
    def search(self, invoice_payment_ids):
        if invoice_payment_ids is None:
            raise InvalidFieldException("Invoice Payment IDs cannot be None.")
        if not isinstance(invoice_payment_ids, list):
            raise InvalidFieldException("Invoice Payment IDs has to be of type list.")
        
        payload = {
            'paymentIds' : invoice_payment_ids
        }

        endpoint = f'/v1/invoices/payment/search'
        response = trolley.Configuration.client(
            self.config).post(endpoint, payload)
        
        invoice_payments = []
                    
        for part in response['invoicePayments']:
            temp_part = trolley.InvoicePaymentPart.factory(part)
            invoice_payments.append(namedtuple("InvoicePaymentPart", temp_part.keys())(*temp_part.values()))
        
        return invoice_payments



from collections import namedtuple
from trolley.exceptions.invalidFieldException import InvalidFieldException
import trolley.configuration
from trolley.types.log import Log
from trolley.types.meta import Meta
from trolley.types.payment import Payment
from trolley.types.recipient import Recipient
from trolley.utils.url_utils import UrlUtils


class RecipientGateway(object):
    """
    Trolley Recipient processor
    Creates and manages transactions
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    """
        Retrieve a recipient
            A recipient_id is required::
            recipient.find('R-fjeracjmuflh')
        """
    def find(self, recipient_id, term=""):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = f'/v1/recipients/{recipient_id}/{term}'
        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint)
        recip = Recipient.factory(response)
        recipient = namedtuple("Recipient", recip.keys())(*recip.values())
        count = 0
        for account in recipient.accounts:
            recipient.accounts[count] = namedtuple(
                "RecipientAccount", account.keys())(*account.values())
            count = count + 1
        return recipient

    """
        Create a recipient
            A body is required::
            recipient.create({"type": "individual", "firstName": "John",
                                    "lastName": "Smith", "email": "jh@edxample.com"})
        """
    def create(self, body):
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        endpoint = '/v1/recipients/'
        response = trolley.configuration.Configuration.client(
            self.config).post(endpoint, body)
        recip = Recipient.factory(response)
        recipient = namedtuple("Recipient", recip.keys())(*recip.values())
        return recipient

    """
        Update a recipient
            A recipient_id and body are required::
            recipient.update('R-fjeracjmuflh', {'firstName': 'tom'})
        """
    def update(self, recipient_id, body):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        if body is None:
            raise InvalidFieldException("Body cannot be None")
        endpoint = f'/v1/recipients/{recipient_id}'
        trolley.configuration.Configuration.client(
            self.config).patch(endpoint, body)
        return True

    """
        Delete a recipient
            A recipient_id is required::
            recipient.delete('R-fjeracjmuflh')
        """
    def delete(self, recipient_id):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = f'/v1/recipients/{recipient_id}'
        trolley.configuration.Configuration.client(
            self.config).delete(endpoint)
        return True
    
    """
        Delete multiple recipients
            A list of recipient_ids is required::
            recipient.delete_multiple([
                'R-fjeracjmuflh',
                'R-ajerasdfuflf'
            ])
        """
    def delete_multiple(self, recipient_ids):
        if recipient_ids is None:
            raise InvalidFieldException("Recipient ids cannot be None.")
        
        endpoint = '/v1/recipients/'
        trolley.configuration.Configuration.client(
            self.config).delete(endpoint, recipient_ids)
        return True

    """
        Retrieve Recipient activity logs, by page
            A recipient_id is required:
            recipient.retrieve_logs('R-fjeracjmuflh')
        """
    def retrieve_logs(self, recipient_id, page=1, pageSize=10):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        
        endpoint = f'/v1/recipients/{recipient_id}/logs?page={page}&pageSize={pageSize}'
        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint)

        logs = []
        count = 0
        for log in response['recipientLogs']:
            temp = Log.factory(log)

            logs.insert(count, namedtuple("Log", temp.keys())(*temp.values()))

            count = count + 1
        
            tempmeta = Meta.factory(response['meta'])
            logs.insert(count,namedtuple("Meta", tempmeta.keys())(*tempmeta.values()))

        return logs
    
    """
        Retrieve all payments of the Recipient
            A recipient_id is required:
            recipient.get_all_payments('R-fjeracjmuflh')
        """
    def get_all_payments(self, recipient_id):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = f'/v1/recipients/{recipient_id}/payments'
        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint)
        
        payments = []
        count = 0
        for payment in response['payments']:
            temppayment = Payment.factory(payment)
            newpayment = namedtuple("Payment", temppayment.keys())(*temppayment.values())
            payments.insert(count, newpayment)
            count = count + 1
        
        tempmeta = Meta.factory(response['meta'])
        payments.insert(count,namedtuple("Meta", tempmeta.keys())(*tempmeta.values()))

        return payments

    """
        Retrieve all offline payments of the Recipient
            A recipient_id is required:
            recipient.get_all_offline_payments('R-fjeracjmuflh')
        """
    def get_all_offline_payments(self, recipient_id):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = f'/v1/recipients/{recipient_id}/offlinePayments'
        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint)
        
        offlinePayments = []
        count = 0
        for offlinePayment in response['offlinePayments']:
            temppayment = Payment.factory(offlinePayment)
            newpayment = namedtuple("Payment", temppayment.keys())(*temppayment.values())
            offlinePayments.insert(count, newpayment)
            count = count + 1
        
        tempmeta = Meta.factory(response['meta'])
        offlinePayments.insert(count,namedtuple("Meta", tempmeta.keys())(*tempmeta.values()))

        return offlinePayments

    """ Search Recipients with a search term.
        This method returns a generator which auto paginates.
        You can use this generator in a foreach loop to sequentially go through all the 
        search results without needing to call this method again.
        
        For accessing specific pages, check the search_by_page() method. 
        """
    def search(self, search=None, name=None, email=None, reference_id=None, start_date=None,
               end_date=None, status=None, compliance_status=None, country=None, payout_method=None, currency=None,
               order_by=None, sort_by=None):
        
        local_vars = UrlUtils.parse(locals())
        
        page = 0
        should_paginate = True
        while should_paginate:
            page+=1
            endpoint = f'/v1/recipients?page={page}&pageSize=10' + (f'&{local_vars}' if(len(local_vars)) else '')
            response = trolley.configuration.Configuration.client(self.config).get(endpoint)
            yield from self.__build_recipients_from_response(response, False)

            if page < response["meta"]["pages"]:
                should_paginate = True
            else:
                should_paginate = False
    
    """ Search Recipients by providing a page number.
        This method returns a list.
        You should use this function when you want to paginate manually.
        recipient.search_by_page(1,10,'test')
     """
    def search_by_page(self, page=1, page_size=10, search=None, name=None, email=None, reference_id=None, start_date=None,
               end_date=None, status=None, compliance_status=None, country=None, payout_method=None, currency=None,
               order_by=None, sort_by=None):
        
        endpoint = '/v1/recipients?' + UrlUtils.parse(locals())
        response = trolley.configuration.Configuration.client(self.config).get(endpoint)
        return self.__build_recipients_from_response(response, True)
        
    def __build_recipients_from_response(self, response, include_meta=False):
        recipients = []
        count = 0
        for recipient in response['recipients']:
            temp = Recipient.factory(recipient)

            recipient = namedtuple("Recipient", temp.keys())(*temp.values())
            recipients.insert(count, recipient)

            count = count + 1
        
        if include_meta:
            tempmeta = Meta.factory(response['meta'])
            recipients.insert(count,namedtuple("Meta", tempmeta.keys())(*tempmeta.values()))

        return recipients



from setuptools import setup

setup (
    name='trolleyhq',
    version='1.0.0',
    packages=["trolley", "trolley.exceptions"],
    package_data={"trolley": ["ssl/*"]},
    install_requires=['requests>=2.31.0'],
    author='Trolley',
    author_email='developer-tools@trolley.com',
    summary='Trolley Python SDK',
    url='https://www.trolley.com',
    license='MIT',
    long_description='A native Python SDK for the Trolley API',
)



from trolley.configuration import Configuration
from trolley.client import Client
from trolley.gateway import Gateway
from trolley.types.recipient import Recipient
from trolley.types.recipient_account import RecipientAccount
from trolley.types.log import Log
from trolley.types.batch import Batch
from trolley.types.batch_summary import BatchSummary
from trolley.types.payment import Payment
from trolley.types.offline_payment import OfflinePayment
from trolley.types.balances import Balances
from trolley.types.invoice import Invoice
from trolley.types.invoice_line import InvoiceLine
from trolley.types.invoice_payment import InvoicePayment
from trolley.types.invoice_payment_part import InvoicePaymentPart


from trolley.exceptions.baseException import BaseException

class AuthenticationException(BaseException):
    """
    An Authentication Exception class.
    """

    def __init__(self, value):
        super().__init__(value)
        self.value = value

    def __str__(self):
        return repr(self.value)



from collections import namedtuple
from trolley.exceptions.invalidFieldException import InvalidFieldException
import trolley.configuration
from trolley.types.balances import Balances


class BalancesGateway(object):
    """
    Trolley Balance module
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    def get_trolley_balance(self):
        return self.get_all_balances("paymentrails")
    
    def get_paypal_balance(self):
        return self.get_all_balances("paypal")

    def get_all_balances(self, term=""):
        if term is None:
            raise InvalidFieldException("Term cannot be None")
        endpoint = '/v1/balances/' + term
        response = trolley.configuration.Configuration.client(
            self.config).get(endpoint)
        
        balances = []
        for balance in response['balances']:
            tempbalance = Balances.factory(balance)
            balances.append(namedtuple("Balances", tempbalance.keys())(*tempbalance.values()))

        return balances
